from junit_xml import TestSuite, TestCase

from secscanner2junit.config import Config
from secscanner2junit.parser import Parser
from secscanner2junit.vulnerability import SastVulnerability


# See following links to learn more about sast scanners and theirs output
# https://docs.gitlab.com/ee/user/application_security/sast/analyzers.html#data-provided-by-analyzers
# https://gitlab.com/gitlab-org/security-products/security-report-schemas/-/blob/master/dist/sast-report-format.json
class SastParser(Parser):
    def __init__(self, report, ts_name, config: Config):
        super().__init__(report, ts_name, config)
        self.p_type = "SAST"

    def parse_vulnerability(self, raw_vulnerability):
        vulnerability = SastVulnerability(raw_vulnerability)

        tc = TestCase(name=vulnerability.get_testcase_name(),
                      classname=self.p_type,
                      file=vulnerability.get_location(),
                      elapsed_sec=1)

        tc.add_failure_info(message=vulnerability.get_description(),
                            output=vulnerability.get_output(),
                            failure_type=vulnerability.get_failure_type())
        return tc

    def parse(self):
        vulnerabilities = self.report['vulnerabilities']
        vulnerabilities = self.config.suppress(vulnerabilities)
        scanners = list(set(vuln['scanner']['name'] for vuln in vulnerabilities))
        testsuites = []

        for scanner in scanners:
            testcases = []
            relevant_vulns = filter(lambda x: x['scanner']['name'] == scanner, vulnerabilities)
            for vuln in relevant_vulns:
                testcases.append(self.parse_vulnerability(vuln))

            testsuites.append(TestSuite(name=self.ts_name + scanner.replace(' ', '-'), test_cases=testcases))

        # If the report was empty, we generate an empty testsuite to return a valid Junit XML file
        if len(testsuites) == 0:
            testsuites.append(TestSuite(name=self.ts_name, test_cases=[]))
        return testsuites



from junit_xml import TestSuite, TestCase

from secscanner2junit.config import Config
from secscanner2junit.parser import Parser
from secscanner2junit.vulnerability import ContainerScanningVulnerability


# See following links to learn more about containers scanners and theirs output
# Example https://gitlab.com/gitlab-examples/security/security-reports/-/blob/master/samples/container-scanning.json
# https://gitlab.com/gitlab-org/security-products/security-report-schemas/-/blob/master/dist/container-scanning-report-format.json
class ContainerScanningParser(Parser):
    def __init__(self, report, ts_name, config: Config):
        super().__init__(report, ts_name, config)
        self.p_type = "ContainerScanning"

    def parse_vulnerability(self, raw_vulnerability):
        vulnerability = ContainerScanningVulnerability(raw_vulnerability)

        tc = TestCase(name=vulnerability.get_testcase_name(),
                      classname=self.p_type,
                      file=vulnerability.get_location(),
                      elapsed_sec=1)

        tc.add_failure_info(message=vulnerability.get_description(),
                            output=vulnerability.get_output(),
                            failure_type=vulnerability.get_failure_type())
        return tc

    def parse(self):
        vulnerabilities = self.report['vulnerabilities']
        vulnerabilities = self.config.suppress(vulnerabilities)

        testsuites = []
        testcases = []

        for raw_vulnerability in vulnerabilities:
            testcases.append(self.parse_vulnerability(raw_vulnerability))

        testsuites.append(TestSuite(name=self.ts_name, test_cases=testcases))
        return testsuites



from yaml import safe_load


class Suppression:
    def __init__(self, type, value):
        self.type = type
        self.value = value

    def __repr__(self):
        return f"Suppression(type={self.type}, value={self.value})"

    def __eq__(self, other):
        if isinstance(other, Suppression):
            return self.type == other.type and self.value == other.value

        return False


class Config:
    def __init__(self, suppressions: list[Suppression]):
        self.suppressions = suppressions

    def __repr__(self):
        return f"Config(suppressions={self.suppressions})"

    def __eq__(self, other):
        if isinstance(other, Config):
            return self.suppressions == other.suppressions

        return False

    def __is_identifier_suppressed(self, identifier):
        for suppression in self.suppressions:
            if suppression.type == identifier['type'] and suppression.value == identifier['value']:
                return True
        return False

    def __is_vulnerability_suppressed(self, vulnerability):
        for identifier in vulnerability['identifiers']:
            if self.__is_identifier_suppressed(identifier):
                return True
        return False

    def suppress(self, vulnerabilities):
        output = list()
        for vulnerability in vulnerabilities:
            if self.__is_vulnerability_suppressed(vulnerability):
                print("Ignoring: " + str(vulnerability))
            else:
                output.append(vulnerability)

        return output


def get_config(path):
    try:
        with open(path) as f:
            yml_dict = safe_load(f)

            suppressions = []
            config_yml = __get_yml_config(yml_dict)
            for suppression in __get_suppressions(config_yml):
                suppressions.append(__get_suppression(suppression))

            config = Config(suppressions)

            print("Loaded config:")
            print(config)

            return config
    except FileNotFoundError as e:
        print("No config found at path: " + path)
        return Config([])


def __get_yml_config(config_yml_dict):
    try:
        return config_yml_dict['sast']
    except TypeError:
        return dict()
    except KeyError:
        return dict()


def __get_suppressions(sast_yml_dict):
    try:
        return sast_yml_dict['suppressions']
    except KeyError:
        return list()


def __get_suppression(suppression_yml_dict):
    try:
        return Suppression(suppression_yml_dict['type'], suppression_yml_dict['value'])
    except KeyError:
        return None



from random import randrange

import yaml

from secscanner2junit.logger import warn


class _Base:
    def __init__(self, raw_object: dict):
        self._raw_object = raw_object

    def _parse_simple_property(self, property_name):
        try:
            return self._raw_object[property_name]
        except KeyError:
            return ''

    def _parse_required_property(self, property_name):
        try:
            return self._raw_object[property_name]
        except KeyError:
            warn("Could not find required property: " + property_name + " in: " + str(self._raw_object))
            return ''

    def _parse_simple_property_list(self, property_name, property_type):
        try:
            elements = self._raw_object[property_name]
            result = []
            for element in elements:
                result.append(property_type(element))
            return result
        except KeyError:
            return []

    def _parse_required_property_list(self, property_name, property_type):
        try:
            elements = self._raw_object[property_name]
            result = []
            for element in elements:
                result.append(property_type(element))
            return result
        except KeyError:
            warn("Could not find required property: " + property_name + " in: " + str(self._raw_object))
            return []


# https://gitlab.com/gitlab-org/security-products/security-report-schemas/-/blob/master/src/security-report-format.json
class _Vulnerability(_Base):
    def __init__(self, raw_vulnerability: dict):
        super().__init__(raw_vulnerability)
        self.__id = self.__parse_id()
        self.__name = super()._parse_simple_property('name')
        self.__description = super()._parse_simple_property('description')
        self.__severity = super()._parse_simple_property('severity')
        self.__solution = super()._parse_simple_property('solution')
        self.__identifiers = super()._parse_required_property_list('identifiers', Identifier)
        self.__links = super()._parse_simple_property_list('links', Link)
        # details
        # tracking
        # flags

    def __repr__(self):
        return f"Vulnerability(raw_vulnerability={self._raw_object})"

    def __parse_id(self):
        try:
            return self._raw_object['id']
        except KeyError:
            warn("Could not find id (use fake instead) for vulnerability: " + str(self._raw_object))
            return 'fake_id_' + str(randrange(1, 10000000))

    def get_testcase_name(self):
        id_part = " (ID: " + self.__id + ")"
        severity_part = " (Severity: " + self.get_severity() + ")"
        if len(self.__name) > 0:
            return self.__name + id_part + severity_part
        else:
            return self.__identifiers[0].get_name() + id_part + severity_part

    def get_location(self):
        pass

    def get_description(self):
        if len(self.__description) > 0:
            return self.__description
        else:
            return "no description"

    def get_failure_type(self):
        return self.__identifiers[0].get_name()

    def get_output(self):
        # Explanation: *width* needs to be set, as yaml otherwise inserts linebreaks after 80 characters which messes
        # up the description and the urls *sort_keys* is True by default, which moves the description to the top and
        # the links to
        return yaml.dump(self._raw_object, width=1000, indent=4, sort_keys=False)

    def get_severity(self):
        if len(self.__severity) > 0:
            return self.__severity
        else:
            return "UNKNOWN"


class SastVulnerability(_Vulnerability):
    def __init__(self, raw_vulnerability: dict):
        super().__init__(raw_vulnerability)
        self.__location = SastLocation(super()._parse_required_property('location'))

    def get_location(self):
        return self.__location.get_location()


class ContainerScanningVulnerability(_Vulnerability):
    def __init__(self, raw_vulnerability: dict):
        super().__init__(raw_vulnerability)
        self.__location = ContainerScanningLocation(super()._parse_required_property('location'))

    def get_location(self):
        return self.__location.get_location()


class SecretsVulnerability(SastVulnerability):
    def __init__(self, raw_vulnerability: dict):
        super().__init__(raw_vulnerability)


class Identifier(_Base):
    def __init__(self, raw_identifier: dict):
        super().__init__(raw_identifier)
        self.__type = super()._parse_required_property('type')
        self.__name = super()._parse_required_property('name')
        self.__url = super()._parse_simple_property('url')
        self.__value = super()._parse_required_property('value')

    def get_name(self):
        return self.__name


class Link(_Base):
    def __init__(self, raw_link: dict):
        super().__init__(raw_link)
        self.__name = super()._parse_simple_property('name')
        self.__url = super()._parse_required_property('url')


class Location(_Base):
    def __init__(self, raw_location: dict):
        super().__init__(raw_location)

    def get_location(self):
        pass


# https://gitlab.com/gitlab-org/security-products/security-report-schemas/-/blob/master/src/sast-report-format.json
class SastLocation(Location):
    def __init__(self, raw_location: dict):
        super().__init__(raw_location)
        self.__file = super()._parse_simple_property('file')
        self.__start_line = super()._parse_simple_property('start_line')
        self.__end_line = super()._parse_simple_property('end_line')
        # class is reserved keyword so property is called class_property
        self.__class_property = super()._parse_simple_property('class')
        self.__method = super()._parse_simple_property('method')

    def get_location(self):
        return self.__file


# https://gitlab.com/gitlab-org/security-products/security-report-schemas/-/blob/master/src/container-scanning-report-format.json
class ContainerScanningLocation(Location):
    def __init__(self, raw_location: dict):
        super().__init__(raw_location)
        self.__dependency = Dependency(super()._parse_required_property('dependency'))
        self.__operating_system = super()._parse_required_property('operating_system')
        self.__image = super()._parse_required_property('image')
        self.__default_branch_image = super()._parse_simple_property('default_branch_image')

    def get_location(self):
        return self.__image


# https://gitlab.com/gitlab-org/security-products/security-report-schemas/-/blob/master/src/security-report-format.json
class Dependency(_Base):
    def __init__(self, raw_dependency: dict):
        super().__init__(raw_dependency)
        self.__package = Package(super()._parse_required_property('package'))
        self.__version = super()._parse_required_property('version')
        self.__iid = super()._parse_simple_property('iid')
        self.__direct = super()._parse_simple_property('direct')
        # dependency_path - TODO in future ?


# https://gitlab.com/gitlab-org/security-products/security-report-schemas/-/blob/master/src/security-report-format.json
class Package(_Base):
    def __init__(self, raw_package: dict):
        super().__init__(raw_package)
        self.__name = super()._parse_required_property('name')



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['secscanner2junit']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=6.0,<7.0', 'junit-xml>=1.9,<2.0', 'pytest>=7.2.0,<8.0.0']

entry_points = \
{'console_scripts': ['ss2ju = secscanner2junit:main']}

setup_kwargs = {
    'name': 'secscanner2junit',
    'version': '0.1.13',
    'description': 'Convert Security Scanner Output to JUnit Format',
    'long_description': '# SecScanner2JUnit\n[![PyPI version](https://badge.fury.io/py/secscanner2junit.svg)](https://badge.fury.io/py/secscanner2junit)\n[![Downloads](https://pepy.tech/badge/secscanner2junit)](https://pepy.tech/project/secscanner2junit)\n\n[![Open in Gitpod](https://gitpod.io/button/open-in-gitpod.svg)](https://gitpod.io/#https://github.com/angrymeir/SecScanner2JUnit)\n\nGitLab offers [security scanning and visualization](https://docs.gitlab.com/ee/user/application_security/) directly via and on their platform.  \nOne nice feature is direct insights on merge requests. However, this feature is only available with the Ultimate tier. To also use this feature on the free tier, one can build around it by taking the security tool output, converting it to the JUnit format, and uploading it as JUnit report.\n\nTo summarize, this tool is for you if:\n- You use GitLab\'s free tier\n- You use Gitlabs [security templates](https://docs.gitlab.com/ee/user/application_security/)\n- You want to easily access security tool output in merge requests\n\nIf you are on the GitLabs Ultimate tier, just use their tooling! No need to mess up your `.gitlab-ci.yml` file. :smile:\n\n## Which scanning types are supported?\nAll scanning types available under the free tier:\n- Secret Scanning\n- Static Application Security Testing (SAST)\n- Container Scanning\n- Infrastructure as Code Scanning\n\n## How to use?\nProcedure:\n1. Overwrite the existing job so that the report can be used by future jobs.  \n2. Convert report\n3. Upload converted report as junit report\n\n### Example for Secret Scanning\nThis example can be used as is.\n```yaml\nstages:\n  - test\n  - convert\n  \n- include:\n  - template: Security/Secret-Detection.gitlab-ci.yml\n  \nsecret_detection:\n  artifacts:\n    paths:\n      - gl-secret-detection-report.json\n    when: always\n    \nsecret_convert:\n  stage: convert\n  dependencies:\n    - secret_detection\n  script:\n    - pip3 install SecScanner2JUnit\n    - ss2ju secrets gl-secret-detection-report.json gl-secret-detection-report.xml\n  artifacts:\n    reports:\n      junit: gl-secret-detection-report.xml\n```\n\n### Example for SAST  \nSince GitLab decides dynamically which scanners to use depending on project languages, it makes sense to first perform a testrun only including the template. This way one can see which jobs are executed and then overwrite them. \n```yaml\nstages:\n  - test\n  - convert\n  \n- include:\n  - template: Security/SAST.gitlab-ci.yml\n  \nsemgrep-sast:\n  after_script:\n    - cp gl-sast-report.json gl-sast-semgrep-report.json\n  artifacts:\n    paths:\n      - gl-sast-semgrep-report.json\n    when: always\n\nbrakeman-sast:\n  after_script:\n    - cp gl-sast-report.json gl-sast-brakeman-report.json\n  artifacts:\n    paths:\n      - gl-sast-brakeman-report.json\n    when: always\n\nsemgrep-sast-convert:\n  stage: convert\n  dependencies:\n    - semgrep-sast\n  script:\n    - pip3 install SecScanner2JUnit\n    - ss2ju sast gl-sast-semgrep-report.json gl-sast-semgrep-report.xml\n  artifacts:\n    reports:\n      junit: gl-sast-semgrep-report.xml\n      \nbrakeman-sast-convert:\n  stage: convert\n  dependencies:\n    - brakeman-sast\n  script:\n    - pip3 install SecScanner2JUnit\n    - ss2ju sast gl-sast-brakeman-report.json gl-sast-brakeman-report.xml\n  artifacts:\n    reports:\n      junit: gl-sast-brakeman-report.xml\n\n```\n\n### Example for Container Scanning\n\n```yaml\n- include:\n  - template: Jobs/Build.gitlab-ci.yml #Build and push the container image\n  - template: Security/Container-Scanning.gitlab-ci.yml #Scan the built image\n\ncontainer_scanning:\n  artifacts:\n    paths:\n      - gl-container-scanning-report-format.json\n    when: always\n\ncontainer_scanning-convert:\n  stage: convert\n  dependencies:\n    - container_scanning\n  script:\n    - pip3 install SecScanner2JUnit\n    - ss2ju container_scanning gl-container-scanning-report.json gl-container-scanning-report.xml\n  artifacts:\n    reports:\n      junit: gl-container-scanning-report.xml\n```\n\n### Suppression\n\nYou can provide a file with suppression which will allow to ignore some vulnerabilities.\n\nYou have to create a file `ss2ju-config.yml` f.e. in `.gitlab` directory which includes:\n\n```yml\nsast:\n  suppressions:\n    - type: "cwe"\n      value: "2555"\n    - type: "find_sec_bugs_type"\n      value: "SPRING_ENDPOINT"\n```\n\nAnd now you can modify execution commands as follows:\n\n```bash\n    - ss2ju sast gl-sast-semgrep-report.json gl-sast-semgrep-report.xml .gitlab/ss2ju-config.yml\n```\n\n\n### Usage with docker\nFor easier usage in CI, `Secscanner2JUnit` is also shipped in a docker container: https://hub.docker.com/r/angrymeir/secscanner2junit  \nIts\' usage is similar to the ways described above:\n```yaml\n...\n\nsecret_convert:\n  stage: convert\n  image:\n    name: angrymeir/secscanner2junit:latest\n    entrypoint: [""]\n  dependencies:\n    - secret_detection\n  script:\n    - ss2ju secrets gl-secret-detection-report.json gl-secret-detection-report.xml\n  artifacts:\n    reports:\n      junit: gl-secret-detection-report.xml\n```\n',
    'author': 'Florian Angermeir',
    'author_email': 'florian.angermeir@tum.de',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/angrymeir/SecScanner2JUnit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



class Parser:
    def __init__(self, report, ts_name, config):
        self.report = report
        self.ts_name = ts_name.rsplit('.', 1)[0].replace('.', '_')
        self.config = config

    def parse(self):
        pass



import enum
import json
import sys
import argparse
import pkg_resources

from junit_xml import to_xml_report_file

from secscanner2junit.config import get_config, Config
from secscanner2junit.container_scanning import ContainerScanningParser
from secscanner2junit.sast import SastParser
from secscanner2junit.secrets import SecretsParser


class ScanType(enum.Enum):
    SECRETS = 'secrets'
    SAST = 'sast'
    CS = 'container_scanning'

    @staticmethod
    def list():
        return list(map(lambda x: x.value, ScanType))


def load_report(input_path):
    with open(input_path) as input_file:
        report = json.load(input_file)
    return report


def save_junit_report(testsuite, output_path):
    with open(output_path, 'w') as output_file:
        to_xml_report_file(output_file, testsuite, prettyprint=True)


def parse_arguments(args):
    arg_parser = argparse.ArgumentParser(description="SecScanner2JUnit: Convert security scanner output to JUnit format.")
    arg_parser.add_argument('--version', action='version', version=pkg_resources.get_distribution('secscanner2junit').version)
    arg_parser.add_argument('activity', choices=ScanType.list())
    arg_parser.add_argument('input_file')
    arg_parser.add_argument('output_file')
    arg_parser.add_argument('config', nargs='?')
    return arg_parser.parse_args(args)


def main(args=None):
    if args is None:
        args = parse_arguments(sys.argv[1:])
    if args.config:
        ss2ju_config = get_config()
    else:
        ss2ju_config = Config([])

    report = load_report(args.input_file)
    if args.activity == ScanType.SECRETS.value:
        parser = SecretsParser(report, args.input_file, ss2ju_config)
    elif args.activity == ScanType.SAST.value:
        parser = SastParser(report, args.input_file, ss2ju_config)
    elif args.activity == ScanType.CS.value:
        parser = ContainerScanningParser(report, args.input_file, ss2ju_config)
    else:
        raise NotImplementedError
    testsuite = parser.parse()
    save_junit_report(testsuite, args.output_file)


if __name__ == '__main__':
    main()
